package com.persagy.apm.diagnose.config;


import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisNode;
import org.springframework.data.redis.connection.RedisPassword;
import org.springframework.data.redis.connection.RedisSentinelConfiguration;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettucePoolingClientConfiguration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.util.StringUtils;
import redis.clients.jedis.JedisPoolConfig;

import java.util.HashSet;
import java.util.Set;


//被@RefreshScope修饰的Bean都是延迟加载的，只有在第一次访问时才会被初始化，刷新Bean也是同理，下次访问时会创建一个信息的对象
@RefreshScope //这里不能省略  不然不能手动触发
@Configuration
public class RedisConfig {


	@Bean
	@ConfigurationProperties(prefix = "spring.redis.pool")
	public JedisPoolConfig getRedisConfig(){
		return new JedisPoolConfig();
	}
	
	/**
	 * 主机地址
	 */
	@Value("${spring.redis.host}")
	private String springRedisHost;
	
	/**
	 * 端口号
	 */
	@Value("${spring.redis.port}")
	private Integer springRedisPort;
	


	/**
	 * 密码(如果有的话)
	 */
	@Value("${spring.redis.password.sentinel}")
	private String springRedisPassword;

	/**
	 * 单机密码
	 */
	@Value("${spring.redis.password.standalone}")
	private String springRedisPasswordStandalone;


	@Value("${spring.redis.database.ten}")
	private Integer springRedisDatabaseTen;
	
	/**
	 * 哨兵集群名称
	 */
	@Value("${spring.redis.sentinel.master}")
	private String springRedisSentinelMaster;
	
	/**
	 * 哨兵集群节点地址
	 */
	@Value("${spring.redis.sentinel.nodes}")
	private String springRedisSentinelNodes;

	
	@Value("${spring.redis.use.cluster}")
	private Boolean redisUseCluster;



	
	/**
	 * 
	 *<p>Description: saas:使用索引为10的Redis库 .</p>
	 *<p>@return RedisTemplate<String,Object>
	 *@since 2019年5月10日: 下午5:18:27
	 *@author gezhanbin
	 */
	@Bean
	public RedisTemplate<String, Object> redisTemplate() {
		RedisTemplate<String, Object> template = new RedisTemplate<>();
		template.setConnectionFactory(redisConnectionFactory(springRedisDatabaseTen));
		template.setDefaultSerializer(new StringRedisSerializer());
		return template;
	}
	
	
	/**
	 * 
	 *<p>Description:  redis连接工厂 .</p> 
	 *<p>@param index redis库的索引
	 *<p>@return RedisConnectionFactory
	 *@since 2019年5月10日: 下午4:49:24
	 *@author gezhanbin
	 */
	private LettuceConnectionFactory redisConnectionFactory(int index) {
		LettuceConnectionFactory redisConnectionFactory;
		if(redisUseCluster) {
			redisConnectionFactory = new LettuceConnectionFactory(redisSentinelConfiguration(index), jedisClientConfiguration());
		} else {
			redisConnectionFactory = new LettuceConnectionFactory(redisStandaloneConfiguration(index), jedisClientConfiguration());
		}
		redisConnectionFactory.afterPropertiesSet();
		return redisConnectionFactory;
	}
	
	
	/**
	 * 
	 *<p>Description:  jedis 客户端 连接池配置.</p> 
	 *<p>@return JedisClientConfiguration
	 *@since 2019年5月10日: 下午4:47:59
	 *@author gezhanbin
	 */
	private LettucePoolingClientConfiguration  jedisClientConfiguration() {
		return LettucePoolingClientConfiguration.builder().poolConfig(getRedisConfig()).build();
	}

	
	
	/**
	 * 
	 *<p>Description:  哨兵模式的redis配置.</p> 
	 *<p>@param index
	 *<p>@return RedisSentinelConfiguration
	 *@since 2019年5月10日: 下午4:20:43
	 *@author gezhanbin
	 */
	private RedisSentinelConfiguration redisSentinelConfiguration(int index) {
		RedisSentinelConfiguration redisSentinelConfiguration = new RedisSentinelConfiguration();
		redisSentinelConfiguration.setDatabase(index);
		redisSentinelConfiguration.setMaster(springRedisSentinelMaster);
		//配置redis的哨兵sentinel
		String[] hosts = StringUtils.delimitedListToStringArray(springRedisSentinelNodes,",");
		Set<RedisNode> redisNodeSet = new HashSet<>();
        for(String redisHost : hosts){
            String[] item = redisHost.split(":");
            String ip = item[0];
            String port = item[1];
            redisNodeSet.add(new RedisNode(ip, Integer.parseInt(port)));
        }
        redisSentinelConfiguration.setSentinels(redisNodeSet);
		if(!StringUtils.isEmpty(springRedisPassword)) {
			redisSentinelConfiguration.setPassword(redisPassword(springRedisPassword));
		}
		return redisSentinelConfiguration;
	}
	
	
	/**
	 * 
	 *<p>Description:  单机模式的redis配置 .</p> 
	 *<p>@param index  redis库的索引
	 *<p>@return RedisStandaloneConfiguration
	 *@since 2019年5月10日: 下午4:05:20
	 *@author gezhanbin
	 */
	private RedisStandaloneConfiguration redisStandaloneConfiguration(int index) {
		RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
		redisStandaloneConfiguration.setDatabase(index);
		redisStandaloneConfiguration.setHostName(springRedisHost);
		redisStandaloneConfiguration.setPort(springRedisPort);
		if(!StringUtils.isEmpty(springRedisPasswordStandalone)) {
			redisStandaloneConfiguration.setPassword(redisPassword(springRedisPasswordStandalone));
		}
		return redisStandaloneConfiguration;
	}

	private RedisPassword redisPassword(String password) {
		return RedisPassword.of(password);
	}
}
